{  Ŀ
    MediaWorks Entertainment Presents 
   

             Red Light  Effect
                By Piotrala
  

  This is fine red  light  effect. It uses
  only 64 colors, not  all 256 as in other
  effects.

}

{$G+}
Program RedLight;

Uses CRT, DOS;

Type
{ Palette array }
  Color = Record
    Red, Green, Blue : Byte;
  End;
  PaletteArray = Array[0..255] Of Color;

{ Virtual screen array }
  ScreenArray = ^ScreenType;
  ScreenType = Array[0..64000] Of Byte;

{ Light array }
  LightArray = ^LightType;
  LightType = Array[0..16383] Of Byte;

Const
  kUp    = 72;  { Keyboard scancodes }
  kDown  = 80;
  kLeft  = 75;
  kRight = 77;
  kEsc   = 1;

Var
  Palette    : PaletteArray;
  Screen     : ScreenArray;
  Image      : ScreenArray;               { Background image                }
  Light      : LightArray;
  i, j, x, y : Word;                      { Counters                        }
  KeyDown    : Array[0..255] Of Boolean;  { Key states                      }
  OldKbdInt  : Pointer;                   { Pointer to old keyboard handler }
  LX, LY     : Word;                      { Position of light middle point  }
  PhongMap   : Pointer;                   { Pointer to a phong map          }
  TempCol    : Byte;                      { Temporary variable              }

Procedure NewKbdInt; Interrupt;
Var
  Key : Byte;
Begin
  Key := Port[$60];
  If Key And $80 = $80 Then
  Begin
    Key := Key And $7F;
    KeyDown[Key] := False
  End
  Else KeyDown[Key] := True;
  Port[$20] := $20
End; { NewKbdInt }

Procedure SetCol(Color, R, G, B : Byte);
Begin
  Port[$3C8] := Color;
  Port[$3C9] := R;
  Port[$3C9] := G;
  Port[$3C9] := B;
End; { SetCol }

Procedure CalcLight;

Function Length(x1, y1, x2, y2 : LongInt) : Word;
Begin
  Length := Round(Sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2)));
End; { Length }

Begin
  GetMem(PhongMap, 65535);
  For y := 0 To 127 Do
  For x := 0 To 127 Do
  Begin
    TempCol := Length(x, y, 128, 128);
    If TempCol > 127 Then TempCol := 127;
    TempCol := 129 - TempCol;
    Mem[Seg(PhongMap^):y Shl 8 + x] := TempCol;
    Mem[Seg(PhongMap^):(255 - y) Shl 8 + x] := TempCol;
    Mem[Seg(PhongMap^):y Shl 8 + (255 - x)] := TempCol;
    Mem[Seg(PhongMap^):(255 - y) Shl 8 + (255 - x)] := TempCol;
  End;
  For y := 0 To 127 Do
  For x := 0 To 127 Do
  Begin
    TempCol := Mem[Seg(PhongMap^):256 * (y * 2) + (x * 2)];
    If TempCol > 63 Then TempCol := 63;
    Light^[128 * y + x] := TempCol;
  End;
  FreeMem(PhongMap, 65535);
End; { CalcLight }

Begin
  Randomize;

  For i := 0 To 255 Do KeyDown[i] := False;
  GetIntVec($09, OldKbdInt);
  SetIntVec($09, @NewKbdInt);

  New(Screen);
  New(Image);
  New(Light);

  FillChar(Screen^[0], 64000, 0);
  FillChar(Image^[0], 64000, 0);
  FillChar(Light^[0], 6400, 0);

  CalcLight;

  Asm
    mov   ax,13h
    int   10h
  End;

{ New palette... }
  For i := 0 To 63 Do SetCol(i, i, 0, 0);
  For i := 64 To 127 Do SetCol(i, 0, i, 0);
  For i := 128 To 191 Do SetCol(i, 0, 0, i);
  For i := 192 To 255 Do SetCol(i, i, i, i);

{ Draw the background }
  For x := 0 To 319 Do
  For y := 0 To 199 Do Image^[320 * y + x] := x Xor y;

  x := 159;  { Put light in the middle of the screen }
  y := 99;

  Repeat
    FillChar(Screen^[0], 64000, 0);
    LX := 0; LY := 0;

  { Calculate the lighten area }
    For i := (x - 64) To (x + 63) Do
    For j := (y - 64) To (y + 63) Do
    Begin
      TempCol := Image^[320 * j + i];
      TempCol := (TempCol * Light^[128 * LY + LX] Div 63) Div 4;
      Screen^[320 * j + i] := TempCol;
      Inc(LX);
      If LX = 128 Then
      Begin
        LX := 0; Inc(LY);
      End;
    End;

  { Show palette on the bottom }
    For i := 0 To 255 Do Screen^[320 * 197 + i] := i;
    For i := 0 To 255 Do Screen^[320 * 198 + i] := i;
    For i := 0 To 255 Do Screen^[320 * 199 + i] := i;

  { An arrow key has been pressed? }
    If KeyDown[kUp] Then If (y - 64) > 0 Then Dec(y);
    If KeyDown[kDown] Then If (y + 63) < 199 Then Inc(y);
    If KeyDown[kLeft] Then If (x - 64) > 0 Then Dec(x);
    If KeyDown[kRight] Then If (x + 63) < 319 Then Inc(x);

  { Put the final frame on the screen }
    Move(Screen^[0], Mem[$A000:0], 64000);
  Until KeyDown[kEsc];

  Dispose(Screen);
  Dispose(Image);
  Dispose(Light);

  TextMode(C80);
  SetIntVec($09, OldKbdInt);

  Writeln('Red Light Effect');
  Writeln('Designed by Piotrala');
  Writeln;
  Writeln('Contact us!');
  Writeln('http://mediaworks.w.interia.pl/');
  Writeln('e-mail: mediaworks@interia.pl');
End. { LIGHT.PAS }